@@ -1,10 +1,10 @@
 package MouseX::ConfigFromFile;
 
-use 5.8.1;
+use 5.008_001;
 use Mouse::Role;
 use MouseX::Types::Path::Class;
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 requires 'get_config_from_file';
 
@@ -17,9 +17,20 @@ has 'configfile' => (
 sub new_with_config {
     my ($class, %params) = @_;
 
-    my $file = defined $params{configfile}
-        ? $params{configfile}
-        : $class->meta->get_attribute('configfile')->default;
+    my $file = defined $params{configfile} ? $params{configfile} : do {
+        my $attr = $class->meta->get_attribute('configfile');
+        if ($attr->has_default) {
+            my $default = $attr->default;
+            ref($default) eq 'CODE' ? $default->($class) : $default;
+        }
+        elsif ($attr->has_builder) {
+            my $builder = $attr->builder;
+            $class->$builder();
+        }
+        else {
+            undef;
+        }
+    };
 
     my %args = (
         defined $file ? %{ $class->get_config_from_file($file) } : (),
